/*          
   TEST PROGRAM 2 FOR 8515 Development Kit
   
   "Digital multimeter"

   it acquires 0-5V samples from ADC channel 1,
   shows them on display lcd (16x2) and on the output leds
   (8bit MSB)
   
   hardware needed: microprocessor,leds, piezo,HD44780 display


*/
#include <avr\io.h>
#include "lcd.h"


void delay(unsigned int p);
void beep(unsigned int t);
void readadc(unsigned char channel,unsigned char mode);

unsigned char adc_lsb;
unsigned char adc_msb;
unsigned long int adc_data=0;
unsigned char adc_d1=0;
unsigned char adc_d2=0;
unsigned char count=0;

int main(void)
{
outp(0xff,DDRC);	//PORT C as output (leds)
outp(0xff,PORTC);	//all pins to 1  (means all leds off)

outp(0x80,DDRD);	//PORT D input except PD7 output(for buzzer)
outp(0x7f,PORTD);	//pull-up , PD7 to 0

outp(0xB0,DDRB);	//PB0-3 IN , PB4-PB7 OUT except MISO (PB6)
outp(0xFF,PORTB);	//outputs to 1, inputs with pull-up
	
outp(0x5C,SPCR);	//SPI Control Register settings:
			//SPIE=0 interrupt disabled
			//SPE=1 SPI enabled 
			//DORD=0 MSB trasmitted first
			//MSTR=1 master mode
			//CPOL=1 clk 1 when idle
			//CPHA=1 clock phase (acquisition pos.edge)
			//SPR1,SPR0=0,0 fclock/4 (2MHz)
			
lcd_init();		//PORT A as output display
lcd_control(1,0,0);	//display on, cursor off, cursor blink off

beep(4000);		//1 second beep 4000 (t*250us)

	
lcd_home();

dprintf("AVR 8515 DEV.KIT");
	
while(1){	
	lcd_goto(0x40);	//GO TO SECOND ROW
	
	readadc(1,1);	//read Channel 1 in single ended mode

	adc_data=(adc_msb<<4)|((adc_lsb>>4)&0x0F);//keep 8 MSB bits
	outp(~adc_data,PORTC); // show 8 MSB with portc leds
	
	adc_data=0;
	adc_data=adc_msb;
	adc_data=((adc_data<<8)&0xFF00)|adc_lsb;
	adc_data=(12036*adc_data)/100000;
	adc_d1=adc_data/100;
	//adc_d2=adc_data%100;
	adc_d2=adc_data-(adc_d1*100);

if (adc_d2<10)
	
	dprintf("%d,0%d V on IN1",adc_d1,adc_d2);
else		
	dprintf("%d,%d V on IN1",adc_d1,adc_d2);
	
	}
}
	
void readadc(unsigned char channel,unsigned char mode)
{
  unsigned char dataw1=0;
  unsigned char dataw2=0;
  unsigned char dataw3=0;
  
  if (channel&0x04) dataw1=(dataw1|0x01);	//set dataw1
  if (mode==1) dataw1=(dataw1|0x02);
  dataw1=(dataw1|0x04);
  
  if (channel&0x02) dataw2=(dataw2|0x80);	//set dataw2
  if (channel&0x01) dataw2=(dataw2|0x40);
  
  cbi(PORTB,4);				//enable slave (SS pin)			
  outp(dataw1,SPDR);			//sends dataw1
  while(bit_is_clear(SPSR,SPIF));	//wait until bit SPIF is set
 
  outp(dataw2,SPDR);			//sends dataw2
  while(bit_is_clear(SPSR,SPIF));	//wait until bit SPIF is set
  adc_msb=inp(SPDR)&0x0F;		//read msb data(ignore 1st 4 bits)
  
  outp(dataw3,SPDR);			//sends dataw3(=0)
  while(bit_is_clear(SPSR,SPIF));	//wait until bit SPIF is set
  sbi(PORTB,4);				//dis. slave (end)
  adc_lsb=inp(SPDR);			//read lsb data word

}	
		

void delay(unsigned int p)   // total delay= p x 5 ms
{
  unsigned int i;
  unsigned char j;
  unsigned char k;
  for(i=0;i<p;i++){			//delay =40000*125ns=5ms
  
    for (j=0;j<100;j++)	{		// (395+1)*100=39600+(4*100)=40000
    	asm("nop");			// 1 cycle
	for (k=0;k<79;k++) asm("nop");	//(4+1)cycles *79= 395
	 }    
	}
}



void beep(unsigned int t)	//buzzer on PD7 ; duration=t*250us (4000=1s)
{
  unsigned int i;
  unsigned char j;
  unsigned char k;

for (i=0;i<t;i++){			//126us*2= 252us period (4KHz)

  sbi(PORTD,7);
    for (j=0;j<2;j++)			//(500+4)*2=1008 *125ns=126us
      for (k=0;k<100;k++) asm("nop");	//(4+1)cycles*100=500
      
  cbi(PORTD,7);
    for (j=0;j<2;j++)			//(500+4)*2=1008 *125ns=126us
      for (k=0;k<100;k++) asm("nop");	//(4+1)cycles*100=500
  }
cbi(PORTD,7);
}
